# =====================================
# generator=datazen
# version=3.1.4
# hash=dd15fcaae00a97a45b4568b47c90f634
# =====================================
"""
quasimoto - Package's default entry-point.
"""

# built-in
import sys

# internal
from quasimoto.entry import main

if __name__ == "__main__":
    sys.exit(main(sys.argv))
